@@ -7,6 +7,13 @@ from collections import Counter
 
 from colorama import Fore, Style
 
+from .better_boto import describe_change_set_with_paginator
+from .stack import ChangeSet
+
+
+if T.TYPE_CHECKING:
+    from boto_session_manager import BotoSesManager
+
 
 class TargetAttributeEnum(enum.Enum):
     PROPERTIES = "Properties"
@@ -79,8 +86,9 @@ class ResourceChange:
     action: str = dataclasses.field()
     resource_type: str = dataclasses.field()
     logical_resource_id: str = dataclasses.field()
-    physical_resource_id: str = dataclasses.field(default=None)
-    replacement: str = dataclasses.field(default=None)
+    physical_resource_id: T.Optional[str] = dataclasses.field(default=None)
+    replacement: T.Optional[str] = dataclasses.field(default=None)
+    change_set_id: T.Optional[str] = dataclasses.field(default=None)
     details: T.List[Detail] = dataclasses.field(default_factory=list)
 
     @classmethod
@@ -91,6 +99,7 @@ class ResourceChange:
             logical_resource_id=dct["LogicalResourceId"],
             physical_resource_id=dct.get("PhysicalResourceId"),
             replacement=dct.get("Replacement"),
+            change_set_id=dct.get("ChangeSetId"),
             details=list(
                 sort_details([Detail.from_dict(d) for d in dct.get("Details", [])])
             ),
@@ -113,23 +122,32 @@ def sort_changes(changes: T.Iterable[ResourceChange]) -> T.Iterable[ResourceChan
     )
 
 
-@dataclasses.dataclass
-class ChangeSet:
-    change_set_id: str = dataclasses.field()
-    change_set_name: str = dataclasses.field()
-    stack_id: str = dataclasses.field()
-    stack_name: str = dataclasses.field()
-    changes: T.List[ResourceChange] = dataclasses.field(default_factory=list)
-
-    @classmethod
-    def from_dict(cls, dct: dict) -> "ChangeSet":
-        return cls(
-            change_set_id=dct["ChangeSetId"],
-            change_set_name=dct["ChangeSetName"],
-            stack_id=dct["StackId"],
-            stack_name=dct["StackName"],
-            changes=[ResourceChange.from_dict(d) for d in dct.get("Changes", [])],
-        )
+# @dataclasses.dataclass
+# class ChangeSet:
+#     change_set_id: str = dataclasses.field()
+#     change_set_name: str = dataclasses.field()
+#     stack_id: str = dataclasses.field()
+#     stack_name: str = dataclasses.field()
+#     description: T.Optional[str] = dataclasses.field(default=None)
+#     changes: T.List[ResourceChange] = dataclasses.field(default_factory=list)
+#
+#     @classmethod
+#     def from_dict(cls, dct: dict) -> "ChangeSet":
+#         """
+#
+#         :param dct: the ``cloudformation_client.describe_change_set`` response.
+#         """
+#         return cls(
+#             change_set_id=dct["ChangeSetId"],
+#             change_set_name=dct["ChangeSetName"],
+#             stack_id=dct["StackId"],
+#             stack_name=dct["StackName"],
+#             description=dct.get("Description"),
+#             changes=[
+#                 ResourceChange.from_dict(d["ResourceChange"])
+#                 for d in dct.get("Changes", [])
+#             ],
+#         )
 
 
 ICON_ADD = "🟢"
@@ -158,7 +176,9 @@ def print_header(msg: str, char: str, length: int, corner_char=""):
 
 
 def visualize_change_set(
-    changes: T.List[dict],
+    change_set: ChangeSet,
+    bsm: T.Optional["BotoSesManager"] = None,
+    include_nested_stack: bool = False,
     _verbose: bool = True,
 ):
     """
@@ -166,13 +186,27 @@ def visualize_change_set(
 
     - https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudformation.html#CloudFormation.Client.describe_change_set
     """
-    resource_change_list: T.List[ResourceChange] = [
-        ResourceChange.from_dict(change["ResourceChange"]) for change in changes
-    ]
-    resource_change_list = list(sort_changes(resource_change_list))
+    print_header(
+        f"    >>> Change for stack {Fore.CYAN}{change_set.stack_name}{Style.RESET_ALL} <<<",
+        " ",
+        80,
+    )
+    print(f"stack id = {Fore.CYAN}{change_set.stack_id}{Style.RESET_ALL}")
+    print(f"change set id = {Fore.CYAN}{change_set.change_set_id}{Style.RESET_ALL}")
+
+    resource_change_list = list(
+        sort_changes(
+            [
+                ResourceChange.from_dict(d["ResourceChange"])
+                for d in change_set.changes
+            ]
+        )
+    )
     action_counter = Counter(
         [resource_change.action for resource_change in resource_change_list]
     )
+
+    # --- Statistics
     if _verbose:  # pragma: no cover
         print_header("Change Set Statistics", "-", 80, "+")
     for change_action in ChangeActionEnum:
@@ -189,10 +223,10 @@ def visualize_change_set(
         print("|")
         print("+" + "-" * 80)
 
+    # --- Details
     if _verbose:  # pragma: no cover
         print_header("Changes", "-", 80, "+")
 
-
     if len(resource_change_list):
         max_logic_resource_id_length = max([
             len(resource_change.logical_resource_id)
@@ -222,3 +256,17 @@ def visualize_change_set(
     if _verbose:  # pragma: no cover
         print("|")
         print("+" + "-" * 80)
+
+    # --- Nested stack
+    if include_nested_stack:
+        for resource_change in resource_change_list:
+            if resource_change.change_set_id is not None:
+                change_set = describe_change_set_with_paginator(
+                    bsm=bsm,
+                    change_set_name=resource_change.change_set_id,
+                )
+                visualize_change_set(
+                    change_set=change_set,
+                    bsm=bsm,
+                    include_nested_stack=include_nested_stack,
+                )
